package org.magmafoundation.bot.autoresponse;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.GuildMessageChannel;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import net.dv8tion.jda.api.interactions.components.ActionRow;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.db.DatabaseManager;
import org.magmafoundation.bot.db.tables.ResponseTable;
import org.magmafoundation.bot.utils.MessageUtils;
import org.magmafoundation.bot.utils.RoleCache;
import org.magmafoundation.bot.utils.btn.BetterButton;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ResponseHandler {

    private static List<ResponseTable.Response> cachedResponses = new ArrayList<>();
    private static Map<GuildMessageChannel, List<Cooldown>> responseCooldowns = new HashMap<>();

    public static void handle(MessageReceivedEvent event) {
        if (cachedResponses == null)
            return;

        if (cachedResponses.size() == 0) {
            if (!Magma.useDatabase)
                return;

            try {
                cachedResponses = (List<ResponseTable.Response>) DatabaseManager.getTable(ResponseTable.class).get();
            } catch (SQLException e) {
                SBLogger.getLogger(ResponseHandler.class).error("Failed to get responses from database", e);
            }
        }

        final GuildMessageChannel channel = event.getChannel().asGuildMessageChannel();
        final Guild guild = event.getGuild();
        final Role staff = guild.getRoleById(Magma.staff_role_id);

        if (handleCooldown(channel))
            return;

        cachedResponses.forEach(response -> {
            String contentRaw = event.getMessage().getContentRaw().toLowerCase();
            String trigger = response.trigger().toLowerCase();

            if (contentRaw.equals(trigger) ||
                    (contentRaw.contains(trigger) &&
                            (contentRaw.contains(trigger + " ") ||
                                    contentRaw.contains(trigger + "?") ||
                                    contentRaw.contains(trigger + "!") ||
                                    contentRaw.contains(trigger + ".") ||
                                    contentRaw.contains(trigger + ",") ||
                                    contentRaw.contains(trigger + ":")))
            ) {
                if (onCooldown(channel, trigger))
                    return;

                EmbedBuilder builder = new EmbedBuilder();
                builder.setTitle(response.title());
                builder.setDescription(response.response().replaceAll("%n", "\n"));
                builder.setColor(Magma.embed_color);
                if (!response.image().isBlank())
                    builder.setThumbnail(response.image());

                long id = MessageUtils.sendMessageGetID(event.getChannel().sendMessageEmbeds(builder.build()));
                if (id == -1)
                    return;

                Button dismiss = new BetterButton()
                        .secondary("magma:dismiss", "\u274C Dismiss")
                        .onEvent(channel.getGuild(), RoleCache.getUsers(staff, guild), onTrigger -> channel.deleteMessageById(id).queue(), complete -> {})
                        .build(id);
                channel.editMessageComponentsById(id, ActionRow.of(dismiss)).queue();
                addCooldown(channel, trigger);
            }
        });
    }

    private static boolean handleCooldown(GuildMessageChannel channel) {
        if (responseCooldowns.containsKey(channel)) {
            List<Cooldown> cooldowns = responseCooldowns.get(channel);
            List<Cooldown> toRemove = new ArrayList<>();
            for (Cooldown cooldown : cooldowns) {
                cooldown.countDown();
                if (cooldown.isOver())
                    toRemove.add(cooldown);
            }
            cooldowns.removeAll(toRemove);
            responseCooldowns.put(channel, cooldowns);
            return !toRemove.isEmpty();
        }
        return false;
    }

    private static boolean onCooldown(GuildMessageChannel channel, String trigger) {
        if (responseCooldowns.containsKey(channel)) {
            List<Cooldown> cooldowns = responseCooldowns.get(channel);
            for (Cooldown cooldown : cooldowns) {
                if (cooldown.getTrigger().equals(trigger))
                    return true;
            }
        }
        return false;
    }

    private static void addCooldown(GuildMessageChannel channel, String trigger) {
        List<Cooldown> cooldowns = responseCooldowns.get(channel);

        if (cooldowns == null)
            cooldowns = new ArrayList<>();

        cooldowns.add(new Cooldown(trigger));
        responseCooldowns.put(channel, cooldowns);
    }

    public static void clearCache() {
        cachedResponses.clear();
    }
}
