package org.magmafoundation.bot.command.base;

import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.channel.middleman.GuildMessageChannel;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.reactions.ReactionHandler;
import org.magmafoundation.bot.utils.btn.ButtonHandler;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

public class CommandManager {

    private static final Map<Guild, ButtonHandler> buttonHandlers = new ConcurrentHashMap<>();
    private static final Map<Guild, ReactionHandler> reactionHandlers = new ConcurrentHashMap<>();

    public static void handle(GuildMessageChannel channel, SlashCommandInteractionEvent event) {
        ICommand command = CommandRegistry.getCommand(event.getName());

        if (event.getGuild() == null || !event.getChannel().canTalk())
            return;

        CommandContext ctx = new CommandContext(channel, event, event.getMember());

        switch (CommandRegistry.doChecks(ctx, command)) {
            case COMMAND_NOT_FOUND -> {
                if (Magma.show_unknown_command_message)
                    event.reply("Unknown command").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            }
            case COMMAND_NO_PERMISSION -> event.reply("You don't have permission to execute this command").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            case COOLDOWN -> event.reply("You are executing commands too fast, try again in 5 seconds").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            case SUCCESS -> {
                assert command != null;
                command.execute(ctx);
            }
        }
    }

    public static ButtonHandler getButtonHandler(Guild guild) {
        if (buttonHandlers.containsKey(guild))
            return buttonHandlers.get(guild);

        ButtonHandler handler = new ButtonHandler();
        buttonHandlers.put(guild, handler);
        return handler;
    }

    public static ReactionHandler getReactionHandler(Guild guild) {
        if (reactionHandlers.containsKey(guild))
            return reactionHandlers.get(guild);

        ReactionHandler reactionHandler = new ReactionHandler();
        reactionHandlers.put(guild, reactionHandler);
        return reactionHandler;
    }

    public static void onGuildJoin(Guild guild) {
        buttonHandlers.put(guild, new ButtonHandler());
        reactionHandlers.put(guild, new ReactionHandler());
    }

    public static void onGuildLeave(Guild guild) {
        buttonHandlers.remove(guild);
        reactionHandlers.remove(guild);
    }
}
