package org.magmafoundation.bot.command.base;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.magmafoundation.bot.command.commands.FINDME;
import org.magmafoundation.bot.core.Magma;
import org.reflections.Reflections;
import org.slf4j.Logger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CommandRegistry {

    private static final Logger LOGGER = SBLogger.getLogger(CommandRegistry.class);
    private static final Map<String, ICommand> commands = new HashMap<>();

    private static final List<CommandData> cachedCommandData = new ArrayList<>();

    public static void init() {
        LOGGER.info("=====Initializing command registry=====");
        Reflections reflections = new Reflections(FINDME.class.getPackage().getName());
        reflections.getSubTypesOf(ICommand.class).forEach(aClass -> {
            try {
                ICommand command = aClass.getConstructor().newInstance();
                addCommand(command);
            } catch (Exception e) {
                LOGGER.error("Command '{}' could not be loaded", aClass.getSimpleName(), e);
            }
        });
        LOGGER.info("=====Command registry initialized=====");
    }

    public static CheckTypes doChecks(CommandContext ctx, ICommand command) {
        if (command == null)
            return Magma.show_unknown_command_message ? CheckTypes.COMMAND_NOT_FOUND : CheckTypes.SUPPRESSED_FAIL;

        Member member = ctx.getMember();

        if (!member.hasPermission(Permission.ADMINISTRATOR) && CheckTypes.onCooldown(member))
            return CheckTypes.COOLDOWN;

        return command.canUserExecute(ctx);
    }

    public static void addCommand(@NotNull ICommand command) {
        if (commands.get(command.name()) != null) {
            LOGGER.warn("Command '{}' already exists", command.name());
            return;
        }

        LOGGER.info("Command '{}' added successfully", command.name());
        commands.put(command.name(), command);
    }

    @Nullable
    public static ICommand getCommand(String name) {
        return commands.getOrDefault(name, null);
    }

    public static List<CommandData> compileCommandData() {
        if (commands.size() != cachedCommandData.size()) {
            cachedCommandData.clear();
            commands.values().forEach(iCommand -> cachedCommandData.add(iCommand.commandData()));
        }
        return cachedCommandData;
    }

      public static Map<String, ICommand> getCommands() {
         return commands;
      }
}
