package org.magmafoundation.bot.command.commands.admin;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.GuildMessageChannel;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.requests.restaction.AuditableRestAction;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.db.DatabaseManager;
import org.magmafoundation.bot.db.tables.BanTable;
import org.magmafoundation.bot.utils.MessageUtils;
import org.magmafoundation.bot.utils.btn.BetterButton;

import java.sql.Date;
import java.sql.SQLException;
import java.util.concurrent.TimeUnit;

public class BanCommand implements ICommand {

    public CheckTypes canUserExecute(CommandContext context) {
        Role supermod = context.getJDA().getRoleById(Magma.super_mod_role_id); //only super mods can use this command
        if (context.getMember().getRoles().contains(supermod))
            return CheckTypes.SUCCESS;
        return CheckTypes.COMMAND_NO_PERMISSION;
    }

    public void execute(CommandContext context) {
        SlashCommandInteractionEvent event = context.getEvent();

        Member author = event.getMember();
        GuildMessageChannel channel = event.getChannel().asGuildMessageChannel();
        Guild guild = channel.getGuild();

        Member member = event.getOption("user").getAsMember();
        final OptionMapping reasonOption = event.getOption("reason");
        String reason = reasonOption != null ? reasonOption.getAsString() : "";

        assert author != null;

        if (member == null) {
            event.reply("This user could not be found").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (!author.canInteract(member)) {
            event.reply("You do not have permission to ban that person").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        Member selfMember = event.getGuild().getSelfMember();

        if (!selfMember.canInteract(member)) {
            event.reply("I do not have permission to ban that person").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        BetterButton[] buttons = new BetterButton[] {
                new BetterButton().success("magma:ban:yes", "Yes")
                        .onEvent(guild, event.getUser(), trigger -> {}, complete -> {
                            AuditableRestAction<Void> ban = guild.ban(member, 7, TimeUnit.DAYS);

                            if (!reason.isEmpty())
                                ban.reason(reason);

                            ban.queue(
                                    success -> {
                                        complete.deferEdit().queue(hook -> hook.editOriginal("`" + member.getUser().getAsTag() + "` has been banned").setComponents().queue());
                                        try {
                                            DatabaseManager.getTable(BanTable.class).add(new BanTable.Ban(member.getIdLong(), event.getUser().getIdLong(), reason, new Date(System.currentTimeMillis())));
                                        } catch (SQLException e) {
                                            SBLogger.getLogger(CheckBanCommand.class).info("Failed to add ban to database");
                                            throw new RuntimeException(e);
                                        }
                                    },
                                    fail -> complete.deferEdit().queue(hook -> hook.editOriginal("`" + member.getUser().getAsTag() + "` could not be banned: " + fail.getMessage()).setComponents().queue())
                            );
                        }),

                new BetterButton().danger("magma:ban:no", "No")
                        .onEvent(guild, event.getUser(), trigger -> {}, complete ->
                                complete.deferEdit().queue(interactionHook -> interactionHook.deleteOriginal().queue())
                        )
        };

        MessageUtils.replyWithButtons(event.reply("Are you sure you want to ban " + member.getUser().getAsTag() + "?"), guild, buttons);
    }

    public String name() {
        return "ban";
    }

    public String help() {
        return "Bans a user from the server";
    }

    public CommandData commandData() {
        return Commands.slash(name(), help())
                .addOption(OptionType.USER, "user", "The user you want to ban", true)
                .addOption(OptionType.STRING, "reason", "The reason you want to ban the user (optional)")
                .setGuildOnly(true);
    }

    public Category category() {
        return Category.ADMIN;
    }
}
