package org.magmafoundation.bot.command.commands.admin;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Activity;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import org.magmafoundation.bot.AppMain;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.db.DatabaseManager;
import org.magmafoundation.bot.db.tables.ActivityTable;

import java.sql.SQLException;

public class ChangePresenceCommand implements ICommand {
   @Override
   public CheckTypes canUserExecute(CommandContext context) {
      Role supermod = context.getJDA().getRoleById(Magma.admin_role_id); //only admins can use this command
      if (context.getMember().getRoles().contains(supermod))
         return CheckTypes.SUCCESS;
      return CheckTypes.COMMAND_NO_PERMISSION;
   }

   @Override
   public void execute(CommandContext context) {
      SlashCommandInteractionEvent event = context.getEvent();

      String activity = event.getOption("activity").getAsString();

      if(activity.equals("reset")){
         activity = "Magma Dev";
      }

      AppMain.getInstance().getPresence().setPresence(Activity.streaming(activity, "https://twitch.tv/magmafoundation"), false);

      try {
         DatabaseManager.getTable(ActivityTable.class).add(activity);
      } catch (SQLException e) {
         SBLogger.getLogger(ChangePresenceCommand.class).error("Error while updating activity in database", e);
      }

      EmbedBuilder embed = new EmbedBuilder()
              .setTitle("Presence changed")
              .setDescription("Presence changed to: " + activity)
              .setColor(Magma.embed_color);
      event.replyEmbeds(embed.build()).setEphemeral(true).queue();
   }

   @Override
   public String name() {
      return "changepresence";
   }

   @Override
   public String help() {
      return "Changes the bot presence";
   }

   @Override
   public CommandData commandData() {
      return Commands.slash(name(), help())
              .addOption(OptionType.STRING, "activity", "The activity to set", true)
              .setGuildOnly(true);
   }

   @Override
   public Category category() {
      return Category.ADMIN;
   }
}
