package org.magmafoundation.bot.command.commands.admin;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.GuildMessageChannel;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.db.DatabaseManager;
import org.magmafoundation.bot.db.tables.BanTable;

import java.sql.SQLException;

public class CheckBanCommand implements ICommand {

   public CheckTypes canUserExecute(CommandContext context) {
      Role supermod = context.getJDA().getRoleById(Magma.super_mod_role_id); //only super mods can use this command
      if (context.getMember().getRoles().contains(supermod))
         return CheckTypes.SUCCESS;
      return CheckTypes.COMMAND_NO_PERMISSION;
   }

   public void execute(CommandContext context) {
      if (!Magma.useDatabase) {
         context.getEvent().reply("Database is disabled").setEphemeral(true).queue();
         return;
      }

      SlashCommandInteractionEvent event = context.getEvent();

      Member author = event.getMember();
      GuildMessageChannel channel = event.getChannel().asGuildMessageChannel();
      Guild guild = channel.getGuild();

      Member member = event.getOption("user").getAsMember();

      try {
         BanTable.Ban ban = (BanTable.Ban) DatabaseManager.getTable(BanTable.class).get(member.getIdLong());
         if (ban == null) {
            EmbedBuilder embed = new EmbedBuilder()
                    .setTitle("Ban Info")
                    .setThumbnail(member.getUser().getAvatarUrl())
                    .setColor(Magma.embed_color)
                    .setDescription(member.getUser().getAsTag() + " is not banned");
            event.replyEmbeds(embed.build()).setEphemeral(true).queue();
            return;
         }

         String reason = ban.reason();
         long bannedBy = ban.moderatorId();
         long userId = ban.userId();

         Member bannedByMember = guild.retrieveMemberById(bannedBy).complete();
         Member bannedMember = guild.retrieveMemberById(userId).complete();

         String bannedByTag = bannedByMember.getUser().getAsTag();
         String bannedTag = bannedMember.getUser().getAsTag();

         String bannedAvatar = bannedMember.getUser().getAvatarUrl();

         EmbedBuilder embed = new EmbedBuilder()
                 .setTitle("Ban Info")
                 .setThumbnail(bannedAvatar)
                 .setColor(Magma.embed_color)
                 .addField("Banned By", bannedByTag, true)
                 .addField("Banned User", bannedTag, true)
                 .addField("Reason", reason, false);

         event.replyEmbeds(embed.build()).setEphemeral(true).queue();
      } catch (SQLException e) {
         SBLogger.getLogger(CheckBanCommand.class).error("Error while checking ban", e);
         event.reply("An error occurred").setEphemeral(true).queue();
      }

   }

   public String name() {
      return "checkban";
   }

   public String help() {
      return "Check if a user is banned";
   }

   public CommandData commandData() {
      return Commands.slash(name(), help())
              .addOption(OptionType.USER, "user", "The user you want to check", true)
              .setGuildOnly(true);
   }

   public Category category() {
      return Category.ADMIN;
   }
}
