package org.magmafoundation.bot.command.commands.admin;

import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.GuildMessageChannel;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;

import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

public class CleanCommand implements ICommand {

    public CheckTypes canUserExecute(CommandContext context) {
        Role staff = context.getJDA().getRoleById(Magma.staff_role_id);
        if (context.getMember().getRoles().contains(staff))
            return CheckTypes.SUCCESS;
        return CheckTypes.COMMAND_NO_PERMISSION;
    }

    public void execute(CommandContext context) {
        SlashCommandInteractionEvent event = context.getEvent();
        GuildMessageChannel channel = context.getChannel();

        int amount = event.getOption("amount").getAsInt();

        if (amount > 1000 || amount < 1) {
            event.reply("Please provide a valid amount of messages to delete (1-1000)").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        channel.getIterableHistory().takeAsync(amount).thenAcceptAsync(messages -> {
            int size = messages.stream().filter(m -> m.getType().canDelete()).toList().size();

            if (size == 0) {
                event.reply("There are no messages to delete").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
                return;
            }

            try {
                List<CompletableFuture<Void>> msgs2delete = channel.purgeMessages(messages).stream().toList();
                CompletableFuture.allOf(msgs2delete.toArray(CompletableFuture[]::new)).get();
            } catch (Exception e) {
                event.reply("Could not delete message(s): " + e.getMessage()).setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
                e.printStackTrace();
            }

            event.reply("Successfully deleted " + size + " message(s)").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
        });
    }

    public String name() {
        return "clean";
    }

    public String help() {
        return "Deletes a specified amount of messages";
    }

    public CommandData commandData() {
        return Commands.slash(name(), help())
                .addOption(OptionType.INTEGER, "amount", "The amount of messages to delete (1-1000)", true)
                .setGuildOnly(true);
    }

    public Category category() {
        return Category.ADMIN;
    }
}
