package org.magmafoundation.bot.command.commands.admin;

import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.GuildMessageChannel;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.requests.restaction.AuditableRestAction;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.utils.MessageUtils;
import org.magmafoundation.bot.utils.btn.BetterButton;

import java.util.concurrent.TimeUnit;

public class KickCommand implements ICommand {

    public CheckTypes canUserExecute(CommandContext context) {
        Role staff = context.getJDA().getRoleById(Magma.staff_role_id);
        if (context.getMember().getRoles().contains(staff))
            return CheckTypes.SUCCESS;
        return CheckTypes.COMMAND_NO_PERMISSION;
    }

    public void execute(CommandContext context) {
        SlashCommandInteractionEvent event = context.getEvent();

        Member author = event.getMember();
        GuildMessageChannel channel = context.getChannel();
        Guild guild = channel.getGuild();

        Member member = event.getOption("user").getAsMember();
        OptionMapping reasonOption = event.getOption("reason");
        String reason = reasonOption != null ? reasonOption.getAsString() : "";

        assert author != null;

        if (member == null) {
            event.reply("This user could not be found").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        if (!author.canInteract(member)) {
            event.reply("You do not have permission to kick that person").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        Member selfMember = event.getGuild().getSelfMember();

        if (!selfMember.canInteract(member)) {
            event.reply("I do not have permission to kick that person").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
            return;
        }

        BetterButton[] buttons = new BetterButton[] {
                new BetterButton().success("magma:kick:yes", "Yes")
                        .onEvent(guild, event.getUser(), trigger -> {}, complete -> {
                            AuditableRestAction<Void> kick = guild.kick(member);

                            if (!reason.isEmpty())
                                kick.reason(reason);

                            kick.queue(
                                    success -> complete.deferEdit().queue(hook -> hook.editOriginal("`" + member.getUser().getAsTag() + "` has been kicked").setComponents().queue()),
                                    fail -> complete.deferEdit().queue(hook -> hook.editOriginal("`" + member.getUser().getAsTag() + "` could not be kicked: " + fail.getMessage()).setComponents().queue())
                            );
                        }),

                new BetterButton().danger("magma:kick:no", "No")
                        .onEvent(guild, event.getUser(), trigger -> {}, complete ->
                                complete.deferEdit().queue(interactionHook -> interactionHook.deleteOriginal().queue())
                        )
        };

        MessageUtils.replyWithButtons(event.reply("Are you sure you want to kick " + member.getUser().getAsTag() + "?"), guild, buttons);
    }

    public String name() {
        return "kick";
    }

    public String help() {
        return "Kicks a user from the server";
    }

    public CommandData commandData() {
        return Commands.slash(name(), help())
                .addOption(OptionType.USER, "user", "The user you want to kick", true)
                .addOption(OptionType.STRING, "reason", "The reason you want to kick the user (optional)")
                .setGuildOnly(true);
    }

    public Category category() {
        return Category.ADMIN;
    }
}
