package org.magmafoundation.bot.command.commands.admin;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.GuildMessageChannel;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandData;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.db.DatabaseManager;
import org.magmafoundation.bot.db.tables.LogUploadTable;
import org.magmafoundation.bot.log.LogUploader;

import java.sql.SQLException;

public class LogUploadingCommand implements ICommand {

    public CheckTypes canUserExecute(CommandContext context) {
        Role supermod = context.getJDA().getRoleById(Magma.super_mod_role_id); //only super mods can use this command
        if (context.getMember().getRoles().contains(supermod))
            return CheckTypes.SUCCESS;
        return CheckTypes.COMMAND_NO_PERMISSION;
    }

    public void execute(CommandContext context) {
        switch (context.getEvent().getSubcommandName()) {
            case "enable" -> {
                LogUploader.enabled = true;
                context.getEvent().reply("Log uploading has been enabled!").queue();
                update(true, context.getChannel());
            }
            case "disable" -> {
                LogUploader.enabled = false;
                context.getEvent().reply("Log uploading has been disabled!").queue();
                update(false, context.getChannel());
            }
            default -> context.getEvent().reply("Please provide a valid subcommand!").setEphemeral(true).queue(interactionHook -> interactionHook.deleteOriginal().queueAfter(5, java.util.concurrent.TimeUnit.SECONDS));
        }
    }

    private void update(boolean enabled, GuildMessageChannel channel) {
        if (!Magma.useDatabase) {
            channel.sendMessage(":warning: Database is not enabled, changes will not persist between restarts!").queue();
            return;
        }

        LogUploadTable.LogUpload upload = new LogUploadTable.LogUpload(enabled);
        try {
            DatabaseManager.getTable(LogUploadTable.class).update(upload);
        } catch (SQLException e) {
            SBLogger.getLogger(getClass()).error("Failed to {} log uploading in database", enabled ? "enable" : "disable", e);
        }
    }

    public String name() {
        return "logupload";
    }

    public String help() {
        return "Control log uploading";
    }

    public CommandData commandData() {
        SubcommandData enable = new SubcommandData("enable", "Enable log uploading");
        SubcommandData disable = new SubcommandData("disable", "Disable log uploading");

        return Commands.slash(name(), help())
                .addSubcommands(enable, disable)
                .setGuildOnly(true);
    }

    public Category category() {
        return Category.ADMIN;
    }
}
