package org.magmafoundation.bot.command.commands.admin;

import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.requests.restaction.AuditableRestAction;
import org.magmafoundation.bot.command.base.*;
import org.magmafoundation.bot.core.Magma;

import static org.magmafoundation.bot.core.Magma.*;

@SuppressWarnings("unused")
public class RoleManegmentCommand implements ICommand {

    @Override
    public CheckTypes canUserExecute(CommandContext context) {
        return CommandUtilities.isMagmaRole(Magma.mod_role_id, context);
    }

    @Override
    public void execute(CommandContext context) {
        final SlashCommandInteractionEvent command = context.getEvent();
        final Guild guild = command.getGuild();
        if (guild == null){
            command.reply("This command is not supported in Direct Messages").setEphemeral(true).queue();
            return;
        }
        if (guild.getIdLong() != guild_id){
            command.reply("This command is not supported in non-Magma servers").setEphemeral(true).queue();
            return;
        }
        final OptionMapping user = command.getOption("user");
        if (user == null){
            command.reply("You must specify a user.").setEphemeral(true).queue();
            return;
        }
        Member member = user.getAsMember();
        if (member == null){
            command.reply("You must specify a user in the Magma guild.").setEphemeral(true).queue();
            return;
        }
        final OptionMapping roleMapping = command.getOption("role");
        if (roleMapping == null){
            command.reply("You must specify a role.").setEphemeral(true).queue();
            return;
        }
        final Role role = guild.getRoleById(roleMapping.getAsString());
        if (role == null){
            command.reply("You the specified role could not be resolved.").setEphemeral(true).queue();
            return;
        }
        final OptionMapping action = command.getOption("action");
        final boolean add = action == null || action.getAsString().equalsIgnoreCase("add");
        final AuditableRestAction<Void> actionResult;
        if (add) actionResult=guild.addRoleToMember(member, role);
        else actionResult=guild.removeRoleFromMember(member, role);
        actionResult.queue(
                success->command.reply((add?"Add":"Remov")+"ed role").setEphemeral(true).queue(),
                throwable -> command.reply("Failed to "+ (add?"add":"remove")+" role").setEphemeral(true).queue()
        );
    }

    @Override
    public String name() {
        return "rolemanagement";
    }

    @Override
    public String help() {
        return "Adds or Removes roles from Users";
    }

    @Override
    public CommandData commandData() {
        final OptionData roles = new OptionData(OptionType.STRING,"role","The role to add or remove from the user",true);
        roles.addChoice("Advertising Blacklist", Magma.ad_blacklist_role_id);
        roles.addChoice("Member", Magma.member_role_id);
        final OptionData actions = new OptionData(OptionType.STRING,"action","Should the role be added or removed (default add)",false);
        actions.addChoice("Add","add");
        actions.addChoice("Remove","remove");

        return Commands.slash(name(), help())
                .setGuildOnly(true)
                .addOptions(new OptionData(OptionType.USER, "user", "The user to add or remove the role from", true), roles, actions);
    }

    @Override
    public Category category() {
        return Category.ADMIN;
    }
}
