package org.magmafoundation.bot.command.commands.admin;

import dev.JustRed23.jdautils.JDAUtilities;
import dev.JustRed23.jdautils.component.interact.SmartReaction;
import dev.JustRed23.jdautils.settings.Setting;
import net.dv8tion.jda.api.entities.*;
import net.dv8tion.jda.api.entities.channel.Channel;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.events.message.react.GenericMessageReactionEvent;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.magmafoundation.bot.AppMain;
import org.magmafoundation.bot.command.base.*;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.utils.MessageUtils;
import org.magmafoundation.bot.utils.btn.BetterButton;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import java.time.OffsetDateTime;
import java.util.concurrent.TimeUnit;

import static org.magmafoundation.bot.core.Magma.*;

@SuppressWarnings("unused")
public class RolesCommand implements ICommand {
    private final Logger LOGGER = LoggerFactory.getLogger(RolesCommand.class);
    private static @Nullable RolesCommand instance;
    private @Nullable String reaction_id;
    private static final String roles_message_id = "roles_message_id";
    private @Nullable Guild magma_guild;
    private @Nullable Message reaction;
    private static final @NotNull MessageEmbed embed;

    static{
        StringBuilder message = new StringBuilder();
        for (int i = 0; i < roles_embed_roles_message_emoji.size(); i++) {
            message.append("<@&");
            message.append(roles_embed_roles.get(i));
            message.append("> - React with: ");
            message.append(roles_embed_roles_message_emoji.get(i));
            message.append("\n");
        }
        message.append("Remove all reaction roles - React with: :x:");
        embed = new MessageEmbed(
                null,
                roles_embed_title,
                message.toString(),
                EmbedType.RICH,
                OffsetDateTime.now(),
                embed_color.getRGB(),
                new MessageEmbed.Thumbnail(roles_embed_url, null, 0,0),
                null,
                null,
                null,
                null,
                null,
                null
        );
    }

    @SuppressWarnings("unused")
    public RolesCommand(){
        if (instance!= null) throw new IllegalStateException("Cannot create multiple instances of a singleton");
        instance = this;
        magma_guild = AppMain.getInstance().getGuildById(guild_id);
        if (JDAUtilities.getGuildSettingManager() != null){
            reaction_id = JDAUtilities.getGuildSettingManager().get(guild_id, roles_message_id).map(Setting::stringValue).orElse(null);
        }else LOGGER.error("Cannot get GuildSettingManager for making verification message persistant. This is needed for the Unverification command.");
        if (reaction_id != null && magma_guild != null){
            final TextChannel channel = magma_guild.getTextChannelById(roles_channel_id);
            if (channel != null) channel.retrieveMessageById(reaction_id).queue(message -> {
                reaction = message;
                populateEvents();
            });
            else LOGGER.error("Could not get the magma channel for verification, because it does not exist according to Discord. ID:"+roles_channel_id);
        }
    }
    @Override
    public CheckTypes canUserExecute(CommandContext context) {
        return CommandUtilities.isMagmaRole(Magma.super_mod_role_id, context);
    }

    private Integer getReactionId(GenericMessageReactionEvent reaction_event){
        final String reaction_emoji = reaction_event.getEmoji().getAsReactionCode();
        for (int i = 0; i < roles_embed_emoji.size(); i++) {
            final String emoji = roles_embed_emoji.get(i);
            if (emoji == null)  continue;
            if (Emoji.fromUnicode(emoji).getAsReactionCode().equals(reaction_emoji)) return i;
        }
        return null;
    }

    private void populateEvents(){
        if (reaction == null) return;

        SmartReaction smartReaction = SmartReaction.create(embed);
        for (final String emoji:roles_embed_emoji) {
            smartReaction = smartReaction.addReaction(Emoji.fromUnicode(emoji));
        }
        smartReaction.addReaction(Emoji.fromUnicode(roles_embed_remove_all_roles_emoji))
                .withListeners(reaction_add->{
                    final Integer reactionId = getReactionId(reaction_add);
                    final String reaction_emoji = reaction_add.getEmoji().getAsReactionCode();
                    final UserSnowflake userSnowflake = UserSnowflake.fromId(reaction_add.getUserIdLong());
                    if (magma_guild == null && reaction_add.getGuild().getIdLong() == guild_id) magma_guild = reaction_add.getGuild();
                    if (reactionId==null){
                        //we have to manually detect the reactions for all roles remove reaction
                        if (reaction_emoji.equals(roles_embed_remove_all_roles_emoji)){
                            //removes all roles from that user, that can be assigned by this autoReaction thing.
                            for (final String roleId:roles_embed_roles) {
                                final Role role = reaction_add.getGuild().getRoleById(roleId);
                                if (role == null || magma_guild == null) continue;
                                magma_guild.removeRoleFromMember(userSnowflake, role).queue();
                            }
                            //removes all reactions from a user
                            reaction_add.retrieveUser().queue(user->{
                                for (final String emoji:roles_embed_emoji) {
                                    reaction_add.retrieveMessage().queue(message1->{
                                        message1.removeReaction(Emoji.fromUnicode(emoji),user).queue();
                                    });
                                }
                            });
                        }
                        reaction_add.retrieveUser().queue(user->reaction_add.getReaction().removeReaction(user).queue());
                        return;
                    }
                    final Role role = magma_guild.getRoleById(roles_embed_roles.get(reactionId));
                    if (role == null){
                        LOGGER.warn("Cannot get role with id:"+roles_embed_roles.get(reactionId)+"in guild: "+magma_guild.getIdLong());
                        return;
                    }
                    magma_guild.addRoleToMember(userSnowflake,role).queue();
                },reaction_delete->{
                    final Integer reactionId = getReactionId(reaction_delete);
                    final UserSnowflake userSnowflake = UserSnowflake.fromId(reaction_delete.getUserIdLong());
                    //removing a non Tracked reaction should not do anything. This also catches the remove all btw, so no logical loops
                    if (reactionId==null) return;
                    if (magma_guild == null && reaction_delete.getGuild().getIdLong() == guild_id) magma_guild = reaction_delete.getGuild();
                    final Role role = magma_guild.getRoleById(roles_embed_roles.get(reactionId));
                    if (role == null){
                        LOGGER.warn("Cannot get role with id:"+roles_embed_roles.get(reactionId)+"in guild: "+magma_guild.getIdLong());
                        return;
                    }
                    magma_guild.removeRoleFromMember(userSnowflake,role).queue();
                }).restore(reaction);
    }

    private void newMessage(@Nonnull Message msg_new){
        for (final String emoji: roles_embed_emoji) {
            msg_new.addReaction(Emoji.fromUnicode(emoji)).queue();
        }
        msg_new.addReaction(Emoji.fromUnicode(roles_embed_remove_all_roles_emoji)).queue();
        reaction = msg_new;
        populateEvents();
        if (JDAUtilities.getGuildSettingManager() == null){
            LOGGER.error("Cannot get GuildSettingManager. Not setting information for persistant message storage");
            return;
        }
        JDAUtilities.getGuildSettingManager().set(guild_id, roles_message_id, msg_new.getIdLong());
    }

    @Override
    public void execute(CommandContext context) {
        final SlashCommandInteractionEvent command = context.getEvent();
        if (roles_embed_roles.size() != roles_embed_roles_message_emoji.size()){
            command.reply("Configuration is wrong. Role embed roles don't match up with Role embed emojis").setEphemeral(true).queue();
            return;
        }
        if (roles_embed_emoji.size() != roles_embed_roles.size()) {
            command.reply("Configuration is wrong. Role embed emojis don't match up with embed reaction emojis").setEphemeral(true).queue();
        }
        final Guild guild = command.getGuild();
        if (guild == null){
            command.reply("This command is not supported in Direct Messages").setEphemeral(true).queue();
            return;
        }
        if (guild.getIdLong() != guild_id){
            command.reply("This command is not supported in non-Magma servers").setEphemeral(true).queue();
            return;
        }
        BetterButton[] btns = new BetterButton[]{
                new BetterButton().danger("magma:verify:yes","Yes")
                        .onEvent(guild,command.getUser(),trigger->{},complete->{
                            if(reaction != null) reaction.delete().queue();
                    complete.deferEdit().queue(msg->{
                        Channel channel = msg.getInteraction().getChannel();
                        if (channel!= null && channel.getId().equals(roles_channel_id)){
                                msg.editOriginal("")
                                        .setEmbeds(embed)
                                        .setComponents()
                                        .queue(this::newMessage);
                        }else{
                            TextChannel rolesChannel = guild.getTextChannelById(roles_channel_id);
                            if (rolesChannel == null){
                                LOGGER.error("Cannot get roles channel with id:"+roles_channel_id+"in guild: "+guild.getIdLong());
                                msg.editOriginal("Cannot get Roles channel").setComponents().queue(_1->msg.deleteOriginal().queueAfter(1, TimeUnit.MINUTES));
                                return;
                            }
                            rolesChannel.sendMessageEmbeds(embed)
                                    .queue(this::newMessage);
                            msg.editOriginal("Sent new Roles message")
                                    .setComponents()
                                    .queue(_1->msg.deleteOriginal()
                                                    .queueAfter(1, TimeUnit.MINUTES)
                                    );
                        }
                    });
                }),
                new BetterButton().success("magma:verify:no","No")
                        .onEvent(guild,command.getUser(),trigger->{},complete->{
                    complete.editMessage("Not replacing roles message").setComponents().queue(msg -> msg.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
                })
        };
        MessageUtils.replyWithButtons(command.reply("Are you sure you want to replace the Roles Message? The old one will be automatically deleted!"), guild, btns);
    }

    @Override
    public String name() {
        return "roles";
    }

    @Override
    public String help() {
        return "Refreshes the AutoReact in the #roles channel";
    }

    @Override
    public CommandData commandData() {
        return Commands.slash(name(), help())
                .setGuildOnly(true);
    }
    @Override
    public Category category() {
        return Category.ADMIN;
    }
}
