package org.magmafoundation.bot.command.commands.admin;

import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;

import static org.magmafoundation.bot.command.base.CommandUtilities.isMagmaRole;
import static org.magmafoundation.bot.core.Magma.*;

public class UnverifyCommand implements ICommand {
    @Override
    public CheckTypes canUserExecute(CommandContext context) {
        if (context.getEvent().getOption("user") == null)
            return CheckTypes.SUCCESS;
        return isMagmaRole(super_mod_role_id, context);
    }

    @Override
    public void execute(CommandContext context) {
        final SlashCommandInteractionEvent command = context.getEvent();
        final Guild guild = command.getGuild();
        if (guild == null){
            command.reply("This command is not supported in Direct Messages").setEphemeral(true).queue();
            return;
        }
        if (guild.getIdLong() != guild_id){
            command.reply("This command is not supported in non-Magma servers").setEphemeral(true).queue();
            return;
        }
        final Role role = guild.getRoleById(verification_role_id);
        if (role == null){
            command.reply("Cannot get role with id:"+verification_role_id+"in guild: "+guild.getIdLong()).setEphemeral(true).queue();
            return;
        }
        try{
            final OptionMapping option = command.getOption("user");
            final User user = option==null?command.getUser():option.getAsUser();
            guild.removeRoleFromMember(user,role).queue();
            {
                VerificationCommand verificationCMD;
                Message message;
                if ((verificationCMD = VerificationCommand.getInstance()) != null &&
                        (message = verificationCMD.getReaction()) != null){
                    message.removeReaction(Emoji.fromUnicode(verification_embed_emoji),user).queue();
                    command.reply("Unverified "+user.getAsMention()).setEphemeral(true).queue();
                }else{
                    command.reply("Removed Verification Role from "+user.getAsMention()+", but there was an error getting the verification message, so the verification reaction of "+user.getAsMention()+" has not been removed.").setEphemeral(true).queue();
                }

            }
        }catch (IllegalStateException e){
            command.reply("There was an error getting the user").setEphemeral(true).queue();
        }
    }

    @Override
    public String name() {
        return "unverify";
    }

    @Override
    public String help() {
        return "Unverifies users, or yourself if no user is specified";
    }

    @Override
    public CommandData commandData() {
        return Commands.slash(name(), help())
                .addOption(OptionType.USER,"user","The user to unverify",false)
                .setGuildOnly(true);
    }
    @Override
    public Category category() {
        return Category.ADMIN;
    }
}
