package org.magmafoundation.bot.command.commands.admin;

import dev.JustRed23.jdautils.JDAUtilities;
import dev.JustRed23.jdautils.component.interact.SmartReaction;
import dev.JustRed23.jdautils.settings.Setting;
import net.dv8tion.jda.api.entities.*;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.magmafoundation.bot.AppMain;
import org.magmafoundation.bot.command.base.*;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.utils.MessageUtils;
import org.magmafoundation.bot.utils.btn.BetterButton;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.time.OffsetDateTime;
import java.util.concurrent.TimeUnit;

import static org.magmafoundation.bot.core.Magma.*;

public class VerificationCommand implements ICommand {
    private static final Logger LOGGER = LoggerFactory.getLogger(VerificationCommand.class);
    private static @Nullable VerificationCommand instance;
    private @Nullable Message reaction;
    private @Nullable String reaction_id;
    private @Nullable String reaction_channel_id;
    private static final String verification_message_id = "verification_message_id";
    private static final String verification_message_chanel_id = "verification_message_chanel_id";
    private @Nullable Guild magma_guild;
    private @Nullable Role verified_role;
    private static final @NotNull MessageEmbed embed = new MessageEmbed(
            null,
            verification_embed_title,
            verification_embed_message,
            EmbedType.RICH,
            OffsetDateTime.now(),
            embed_color.getRGB(),
            new MessageEmbed.Thumbnail(verification_embed_url, null, 0,0),
            null,
            null,
            null,
            null,
            null,
            null
    );

    @SuppressWarnings("unused")
    public VerificationCommand(){
        if (instance != null)
            throw new IllegalStateException("Cannot create multiple instances of a singleton");
        instance = this;
        magma_guild = AppMain.getInstance().getGuildById(guild_id);
        if (JDAUtilities.getGuildSettingManager() != null){
            reaction_id = JDAUtilities.getGuildSettingManager().get(guild_id, verification_message_id).map(Setting::stringValue).orElse(null);
            reaction_channel_id = JDAUtilities.getGuildSettingManager().get(guild_id, verification_message_chanel_id).map(Setting::stringValue).orElse(null);
        }else LOGGER.error("Cannot get GuildSettingManager for making verification message persistant. This is needed for the Unverification command.");
        if (reaction_id != null && reaction_channel_id!=null && magma_guild != null){
            final TextChannel channel = magma_guild.getTextChannelById(reaction_channel_id);
            if (channel != null) channel.retrieveMessageById(reaction_id).queue(message -> {
                reaction = message;
                populateRole();
                populateEvents();
            });
            else LOGGER.error("Could not get the magma channel for verification, because it does not exist according to Discord. ID:"+reaction_channel_id);
        }
    }
    private void populateRole(){
        if(verified_role!=null) return;
        if (magma_guild!=null) verified_role = magma_guild.getRoleById(verification_role_id);
    }

    private void populateEvents(){
        if (reaction == null) return;
        SmartReaction.create(embed)
                        .addReaction(Emoji.fromUnicode(verification_embed_emoji))
                                .withListeners(role_add->{
                                    if (!role_add.getEmoji().getAsReactionCode().equals(Emoji.fromUnicode(verification_embed_emoji).getAsReactionCode())){
                                        role_add.retrieveUser().queue(user -> role_add.getReaction().removeReaction(user).queue());
                                        return;
                                    }
                                    if (role_add.getGuild().getIdLong() == Magma.guild_id) magma_guild = role_add.getGuild();
                                    populateRole();
                                    if (verified_role!=null && magma_guild != null) role_add.getGuild().addRoleToMember(UserSnowflake.fromId(role_add.getUserIdLong()),verified_role).queue();
                                    else LOGGER.error("Reaction was added on Verification message, but the role or guild was null.");
                                },role_delete->{
                                    if (!role_delete.getEmoji().getAsReactionCode().equals(Emoji.fromUnicode(verification_embed_emoji).getAsReactionCode())) return;
                                    if (role_delete.getGuild().getIdLong() == Magma.guild_id) magma_guild = role_delete.getGuild();
                                    populateRole();
                                    if (verified_role!=null && magma_guild != null) magma_guild.removeRoleFromMember(UserSnowflake.fromId(role_delete.getUserIdLong()),verified_role).queue();
                                    else LOGGER.error("Reaction was added on Verification message, but the role or guild was null.");
                                }).restore(reaction);
    }

    public static @Nullable VerificationCommand getInstance() {
        return instance;
    }
    public @Nullable Message getReaction() {
        if (reaction!=null) return reaction;
        if (reaction_id!=null && reaction_channel_id!=null){
            Guild guild;
            TextChannel channel;
            if ((guild = AppMain.getInstance().getGuildById(guild_id)) != null &&
                    (channel=guild.getChannelById(TextChannel.class,reaction_channel_id))!= null)
                        reaction = channel.retrieveMessageById(reaction_id).complete();
            else {
                LOGGER.error("Cannot get guild or channel for verification message.");
                return null;
            }
        }
        return null;
    }

    @Override
    public CheckTypes canUserExecute(CommandContext context) {
        return CommandUtilities.isMagmaRole(Magma.super_mod_role_id, context);
    }

    @Override
    public void execute(CommandContext context) {
        final SlashCommandInteractionEvent command = context.getEvent();
        final Guild guild = command.getGuild();
        if (guild == null){
            command.reply("This command is not supported in Direct Messages").setEphemeral(true).queue();
            return;
        }
        if (guild.getIdLong() != guild_id){
            command.reply("This command is not supported in non-Magma servers").setEphemeral(true).queue();
            return;
        }
        if (magma_guild == null) magma_guild = guild;
        populateRole();
        if (verified_role == null){
            command.reply("Cannot get role with id:"+verification_role_id+"in guild: "+guild.getIdLong()).setEphemeral(true).queue();
            return;
        }
        BetterButton[] btns = new BetterButton[]{
                new BetterButton().danger("magma:verify:yes","Yes")
                        .onEvent(guild,command.getUser(),trigger->{},complete->{
                            if (reaction != null) reaction.delete().queue();
                            complete.deferEdit().queue(msg->{
                                msg.editOriginal("")
                                    .setEmbeds(embed)
                                    .setComponents()
                                    .queue(msg_new->{
                                        msg_new.addReaction(Emoji.fromUnicode(verification_embed_emoji)).queue();
                                        reaction = msg_new;
                                        populateEvents();
                                        if (JDAUtilities.getGuildSettingManager() == null){
                                            LOGGER.error("Cannot get GuildSettingManager. Not setting information for persistant message storage");
                                            return;
                                        }
                                        JDAUtilities.getGuildSettingManager().set(guild_id, verification_message_chanel_id, msg_new.getChannel().getIdLong());
                                        JDAUtilities.getGuildSettingManager().set(guild_id, verification_message_id, msg_new.getIdLong());
                                    });
                                }
                            );
                }),
                new BetterButton().success("magma:verify:no","No")
                        .onEvent(guild,command.getUser(),trigger->{},complete->{
                    complete.editMessage("Not replacing Verification message").setComponents().queue(msg -> msg.deleteOriginal().queueAfter(5, TimeUnit.SECONDS));
                })
        };
        MessageUtils.replyWithButtons(command.reply("Are you sure you want to replace the Verification Message? The old one will be automatically deleted!"), guild, btns);
    }

    @Override
    public String name() {
        return "verification";
    }

    @Override
    public String help() {
        return "Creates a message for verification purposes";
    }

    @Override
    public CommandData commandData() {
        return Commands.slash(name(), help())
                .setGuildOnly(true);
    }
    @Override
    public Category category() {
        return Category.ADMIN;
    }

}
