package org.magmafoundation.bot.command.commands.info;

import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.entities.SelfUser;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import org.magmafoundation.bot.AppMain;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.utils.TimeUtils;

public class AboutCommand implements ICommand {

    public CheckTypes canUserExecute(CommandContext context) {
        return CheckTypes.SUCCESS;
    }

    public void execute(CommandContext context) {
        JDA jda = context.getJDA();
        User owner = jda.retrieveUserById(Magma.owner_id).complete();
        SelfUser self = jda.getSelfUser();

        long uptimeMs = (System.currentTimeMillis() - Magma.start_time);
        String uptime = TimeUtils.millisToTime(uptimeMs);

        EmbedBuilder builder = new EmbedBuilder();
        builder.setColor(Magma.embed_color);
        builder.setAuthor(self.getName() + " " + AppMain.getVersion().getVersion(), Magma.repo_url + "/-/commit/" + AppMain.getVersion().gitHash());
        builder.setThumbnail(self.getAvatarUrl());
        builder.setFooter("Made by " + owner.getAsTag(), owner.getEffectiveAvatarUrl());

        builder.addField("Total users", "" + context.getJDA().getUserCache().size(), true);
        builder.addField("Uptime", uptime, false);
        builder.addField("Built", AppMain.getVersion().buildTime().toString(), false);

        context.getEvent().replyEmbeds(builder.build()).queue();
    }

    public String name() {
        return "about";
    }

    public String help() {
        return "Shows information about the bot";
    }

    public CommandData commandData() {
        return Commands.slash(name(), help()).setGuildOnly(true);
    }

    public Category category() {
        return Category.INFO;
    }
}
