package org.magmafoundation.bot.command.commands.info;

import dev.JustRed23.stonebrick.exceptions.NetRequestException;
import dev.JustRed23.stonebrick.net.Callback;
import dev.JustRed23.stonebrick.net.NetworkManager;
import dev.JustRed23.stonebrick.net.Request;
import dev.JustRed23.stonebrick.net.Response;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import org.json.JSONObject;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;

public class MagmaStatsCommand implements ICommand {

   @Override
   public CheckTypes canUserExecute(CommandContext context) {
      return CheckTypes.SUCCESS;
   }

   @Override
   public void execute(CommandContext context) {
      String statsAPI = "https://api.magmafoundation.org/api/v2/stats";
      NetworkManager.get(statsAPI).async(new Callback() {
         @Override
         public void response(Request request, Response response, NetRequestException e) {
            if (e != null) {
               context.getEvent().reply("An error occurred while fetching the stats").queue();
               return;
            }

            JSONObject json = response.asJSONObject();
            int totalIssues = json.getInt("total_issues");
            int playerCount = json.getInt("player_count");
            int serverCount = json.getInt("server_count");
            int openIssues = json.getInt("open_issues");
            int closedIssues = json.getInt("closed_issues");

            String embedTitle = "Magma Statistics "+ Magma.magma_emote;

            EmbedBuilder embed = new EmbedBuilder();
            embed.setTitle(embedTitle);

            embed.addField("Open Issues", String.valueOf(openIssues), true);
            embed.addField("Closed Issues", String.valueOf(closedIssues), true);
            embed.addField("Total Issues", String.valueOf(totalIssues), false);
            embed.addField("Player Count", String.valueOf(playerCount), true);
            embed.addField("Server Count", String.valueOf(serverCount), true);

            embed.setColor(Magma.embed_color);
            embed.setFooter("https://api.magmafoundation.org/api/v2/stats");

            context.getEvent().replyEmbeds(embed.build()).queue();
         }
      });

   }

   @Override
   public String name() {
      return "stats";
   }

   @Override
   public String help() {
      return "Display statistics such as player count, server count, and more";
   }

   @Override
   public CommandData commandData() {
      return Commands.slash(name(), help()).setGuildOnly(true);
   }

   @Override
   public Category category() {
      return Category.INFO;
   }
}
