package org.magmafoundation.bot.command.commands.info;

import net.dv8tion.jda.api.entities.EmbedType;
import net.dv8tion.jda.api.entities.MessageEmbed;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.requests.restaction.CacheRestAction;
import net.dv8tion.jda.api.utils.TimeUtil;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;

import java.awt.Color;
import java.time.OffsetDateTime;
import java.util.Arrays;
import java.util.stream.Collectors;

import static org.magmafoundation.bot.core.Magma.embed_color;

public class WhoCommand implements ICommand {
    @Override
    public CheckTypes canUserExecute(CommandContext context) {
        return CheckTypes.SUCCESS;
    }

    @Override
    public void execute(CommandContext context) {
        final SlashCommandInteractionEvent command = context.getEvent();
        final OptionMapping optionMapping = command.getOption("user");
        if (optionMapping == null){
            command.reply("No user specified").setEphemeral(true).queue();
            return;
        }
        CacheRestAction<User> userFuture;
        try {
            userFuture = command.getJDA().retrieveUserById(optionMapping.getAsString());
        } catch (NumberFormatException e){
            command.reply("Invalid user id").setEphemeral(true).queue();
            return;
        }
        userFuture.queue(user-> {
            user.retrieveProfile().queue(profile -> {
                final Color color = profile.getAccentColor();

                command.replyEmbeds(new MessageEmbed(null,
                        "User Information",
                        null,
                        EmbedType.RICH,
                        OffsetDateTime.now(),
                        embed_color.getRGB(),
                        new MessageEmbed.Thumbnail(user.getEffectiveAvatarUrl(), null, 0, 0),
                        null,
                        null,
                        null,
                        null,
                        new MessageEmbed.ImageInfo(profile.getBannerUrl(), null, 0, 0),
                        Arrays.asList(
                                new MessageEmbed.Field("Username", user.getAsTag(), true),
                                new MessageEmbed.Field("ID", user.getId(), true),
                                new MessageEmbed.Field("Created", TimeUtil.getDateTimeString(user.getTimeCreated()), true),
                                new MessageEmbed.Field("Flags", user.getFlags().stream().map(User.UserFlag::getName).collect(Collectors.joining(", ")), true),
                                new MessageEmbed.Field("Bot/System", (user.isBot() ? "Yes" : "No")+"/"+(user.isSystem() ? "Yes" : "No"), true),
                                new MessageEmbed.Field("Accent Color", color == null ? "None" : "0x"+Integer.toHexString(color.getRGB()), true)
                        ))).queue();
            });
        });
    }

    @Override
    public String name() {
        return "who";
    }

    @Override
    public String help() {
        return "Gets information about a user";
    }

    @Override
    public CommandData commandData() {
        return Commands.slash(name(), help())
                .addOption(OptionType.USER,"user","The user id to get information about",true);
    }

    @Override
    public Category category() {
        return Category.INFO;
    }
}
