package org.magmafoundation.bot.command.commands.utility;

import dev.JustRed23.stonebrick.exceptions.NetRequestException;
import dev.JustRed23.stonebrick.net.Callback;
import dev.JustRed23.stonebrick.net.NetworkManager;
import dev.JustRed23.stonebrick.net.Request;
import dev.JustRed23.stonebrick.net.Response;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import org.json.JSONObject;
import org.magmafoundation.bot.command.base.Category;
import org.magmafoundation.bot.command.base.CheckTypes;
import org.magmafoundation.bot.command.base.CommandContext;
import org.magmafoundation.bot.command.base.ICommand;
import org.magmafoundation.bot.core.Magma;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class MagmaVersionCommand implements ICommand {


   @Override
   public CheckTypes canUserExecute(CommandContext context) {
      return CheckTypes.SUCCESS;
   }

   @Override
   public void execute(CommandContext context) {
      SlashCommandInteractionEvent event = context.getEvent();

      String releaseAPI = "https://api.magmafoundation.org/api/v2/%s/latest";

      String version = event.getOption("version").getAsString();

      NetworkManager.get(releaseAPI.formatted(version)).async(new Callback() {
         @Override
         public void response(Request request, Response response, NetRequestException e) {
            if (e != null) {
               event.reply("An error occurred while fetching the stats").queue();
               return;
            }

            JSONObject array = response.asJSONObject();

            if (array.isEmpty()) {
               event.reply("There are currently no releases for this version").queue();
               return;
            }

            if (array.has("error")) {
               event.reply(array.getString("error")).queue();
               return;
            }

            String url = "[click here](%s)".formatted(array.getString("link"));
            String tag = array.getString("tag_name");

            Date date = null;
            try {
               date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(array.getString("created_at"));
            } catch (ParseException ex) {
               throw new RuntimeException(ex);
            }
            String created_at = new SimpleDateFormat("dd/MM/yyyy, Ka").format(date);
            String archived = array.getBoolean("archived") ? "Yes" : "No";
            String git_commit_url = "[click here](%s)".formatted(array.getString("git_commit_url"));

            EmbedBuilder embed = new EmbedBuilder()
                    .setTitle("Magma %s Release".formatted(version))
                    .addField("Tag", tag, true)
                    .addField("Created At", created_at, true)
                    .addField("EOL", archived, true)
                    .addField("Download", url, true)
                    .addField("Git Commit", git_commit_url, true)
                    .setColor(Magma.embed_color);

            event.replyEmbeds(embed.build()).queue();

         }
      });

   }

   @Override
   public String name() {
      return "releases";
   }

   @Override
   public String help() {
      return "Shows the latest releases of Magma";
   }

   @Override
   public CommandData commandData() {
      OptionData version = new OptionData(OptionType.STRING, "version", "The version to get info about", true)
              .addChoice("1.12.2", "1.12.2")
              .addChoice("1.16.5", "1.16.5")
              .addChoice("1.18.2", "1.18.2")
              .addChoice("1.19.3", "1.19.3");

      return Commands.slash(name(), help())
              .addOptions(version)
              .setGuildOnly(true);
   }

   @Override
   public Category category() {
      return Category.UTILITY;
   }
}
