package org.magmafoundation.bot.db;

import dev.JustRed23.stonebrick.log.SBLogger;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.magmafoundation.bot.core.DBConnection;
import org.magmafoundation.bot.core.Magma;
import org.reflections.Reflections;
import org.slf4j.Logger;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;


public final class DatabaseManager {

   private static Connection connection;
   private static final Logger LOGGER = SBLogger.getLogger(DatabaseManager.class);

   private static final boolean enabled = Magma.useDatabase;

   private static final List<Table<?>> tables = new ArrayList<>();

   public static void init() {
      if (!enabled) return;

      try {
         getConnection();
         createTables();
      } catch (SQLException e) {
         throw new RuntimeException(e);
      }
   }

   public static Connection getConnection() {
      if (!enabled) return null;
      try {
         connect();
      } catch (SQLException e) {
         //try again once
         try {
            connect();
         } catch (SQLException e1) {
            LOGGER.error("Failed to connect to database", e1);
         }
      }
      return connection;
   }

   private static void connect() throws SQLException {
      String dbURL = "jdbc:mariadb://" + DBConnection.host + ":" + DBConnection.port + "/" + DBConnection.database + "?connectTimeout=0&socketTimeout=0&autoReconnect=true";
      if (connection == null || connection.isClosed()) {
         LOGGER.info("Connecting to database...");
         connection = DriverManager.getConnection(dbURL, DBConnection.username, DBConnection.password);
         LOGGER.info("Connected to database");
      }
   }

   private static void createTables() throws SQLException {
      LOGGER.info("Creating tables...");
      Reflections reflections = new Reflections(Table.class.getPackage().getName() + ".tables");
      for (Class<? extends Table> table : reflections.getSubTypesOf(Table.class)) {
         try {
            Table<?> instance = table.getConstructor().newInstance();
            instance.createTable();
            tables.add(instance);
         } catch (Exception e) {
            LOGGER.error("Error while creating table " + table.getSimpleName(), e);
         }
      }
   }

   @Nullable
   public static Table<?> getTable(@NotNull Class<?> clazz) {
      if (!Magma.useDatabase)
         return new NOOPTable();
      return tables.stream().filter(table -> table.getClass().equals(clazz)).findFirst().orElse(null);
   }
}
