package org.magmafoundation.bot.db.tables;

import org.magmafoundation.bot.db.Table;

import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import static org.magmafoundation.bot.db.DatabaseManager.getConnection;

public class BanTable extends Table<BanTable.Ban> {

    public record Ban(long userId, long moderatorId, String reason, Date createdAt) {}

    public void createTable() throws SQLException {
        StringBuilder sql = new StringBuilder();
        sql.append("CREATE TABLE IF NOT EXISTS bans (");
        sql.append("  id INTEGER PRIMARY KEY AUTO_INCREMENT,");
        sql.append("  user_id VARCHAR(100) NOT NULL,");
        sql.append("  moderator_id VARCHAR(100) NOT NULL,");
        sql.append("  reason TEXT,");
        sql.append("  created_at DATETIME DEFAULT CURRENT_TIMESTAMP");
        sql.append(");");
        getConnection().createStatement().executeUpdate(sql.toString());
    }

    public Ban get(Object... args) throws SQLException {
        long userID = (long) args[0];

        PreparedStatement statement = getConnection().prepareStatement("SELECT * FROM bans WHERE user_id = ?");
        statement.setLong(1, userID);
        ResultSet resultSet = statement.executeQuery();
        if (resultSet.next()) {
            return new Ban(resultSet.getLong("user_id"), resultSet.getLong("moderator_id"), resultSet.getString("reason"), resultSet.getDate("created_at"));
        }
        return null;
    }

    public void add(Object... args) throws SQLException {
        Ban ban = (Ban) args[0];

        if (ban.reason.equals("")) {
            ban = new Ban(ban.userId, ban.moderatorId, "No reason provided", ban.createdAt);
        }

        PreparedStatement statement = getConnection().prepareStatement("INSERT INTO bans (user_id, moderator_id, reason, created_at) VALUES (?, ?, ?, ?)");
        statement.setLong(1, ban.userId);
        statement.setLong(2, ban.moderatorId);
        statement.setString(3, ban.reason);
        statement.setDate(4, ban.createdAt);
        statement.executeUpdate();
    }
}
