package org.magmafoundation.bot.db.tables;

import org.magmafoundation.bot.db.Table;

import java.sql.PreparedStatement;
import java.sql.SQLException;

import static org.magmafoundation.bot.db.DatabaseManager.getConnection;

public class LogUploadTable extends Table<LogUploadTable.LogUpload> {

    public record LogUpload(boolean enabled) {}

    public void createTable() throws SQLException {
        StringBuilder sql = new StringBuilder();
        sql.append("CREATE TABLE IF NOT EXISTS log_uploads (");
        sql.append("  id INTEGER PRIMARY KEY AUTO_INCREMENT,");
        sql.append("  enabled BOOLEAN DEFAULT TRUE");
        sql.append(");");
        getConnection().createStatement().executeUpdate(sql.toString());
        add(new LogUpload(true));
    }

    public LogUpload get(Object... args) throws SQLException {
        PreparedStatement statement = getConnection().prepareStatement("SELECT * FROM log_uploads ORDER BY id DESC LIMIT 1");
        var result = statement.executeQuery();
        if (result.next()) {
            return new LogUpload(result.getBoolean("enabled"));
        }
        return new LogUpload(true);
    }

    public void add(Object... args) throws SQLException {
        LogUpload logUpload = (LogUpload) args[0];

        PreparedStatement statement = getConnection().prepareStatement("INSERT INTO log_uploads (enabled) VALUES (?)");
        statement.setBoolean(1, logUpload.enabled);
        statement.executeUpdate();
    }

    public void update(Object... args) throws SQLException {
        LogUpload logUpload = (LogUpload) args[0];

        PreparedStatement statement = getConnection().prepareStatement("UPDATE log_uploads SET enabled = ? WHERE id = (SELECT MAX(id) FROM log_uploads)");
        statement.setBoolean(1, logUpload.enabled);
        statement.executeUpdate();
    }
}
