package org.magmafoundation.bot.db.tables;

import org.magmafoundation.bot.db.Table;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import static org.magmafoundation.bot.db.DatabaseManager.getConnection;

public class ResponseTable extends Table<List<ResponseTable.Response>> {

    public record Response(String trigger, String title, String response, String image) {}

    protected void createTable() throws SQLException {
        StringBuilder sql = new StringBuilder();
        sql.append("CREATE TABLE IF NOT EXISTS responses (");
        sql.append("  id INTEGER PRIMARY KEY AUTO_INCREMENT,");
        sql.append("  trig VARCHAR(100) NOT NULL,");
        sql.append("  title VARCHAR(100) NOT NULL,");
        sql.append("  image VARCHAR(255) NOT NULL,");
        sql.append("  response TEXT NOT NULL");
        sql.append(");");
        getConnection().createStatement().executeUpdate(sql.toString());
    }

    public List<Response> get(Object... args) throws SQLException {
        String sql = "SELECT * FROM responses";
        PreparedStatement statement = getConnection().prepareStatement(sql);
        ResultSet result = statement.executeQuery();

        List<Response> responses = new ArrayList<>();
        while (result.next()) {
            responses.add(new Response(
                    result.getString("trig"),
                    result.getString("title"),
                    result.getString("response"),
                    result.getString("image")
            ));
        }

        return responses;
    }

    public void add(Object... args) throws SQLException {
        Response response = (Response) args[0];

        PreparedStatement statement = getConnection().prepareStatement("INSERT INTO responses (trig, title, response, image) VALUES (?, ?, ?, ?)");
        statement.setString(1, response.trigger());
        statement.setString(2, response.title());
        statement.setString(3, response.response());
        statement.setString(4, response.image());
        statement.executeUpdate();
    }

    public void remove(Object... args) throws SQLException {
        String trigger = (String) args[0];

        PreparedStatement statement = getConnection().prepareStatement("DELETE FROM responses WHERE trig = ?");
        statement.setString(1, trigger);
        statement.executeUpdate();
    }
}
