package org.magmafoundation.bot.event;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.events.guild.GuildJoinEvent;
import net.dv8tion.jda.api.events.guild.GuildLeaveEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRoleAddEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRoleRemoveEvent;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.events.message.MessageDeleteEvent;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import net.dv8tion.jda.api.events.message.react.MessageReactionAddEvent;
import net.dv8tion.jda.api.events.role.RoleDeleteEvent;
import net.dv8tion.jda.api.events.session.ReadyEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import org.jetbrains.annotations.NotNull;
import org.magmafoundation.bot.AppMain;
import org.magmafoundation.bot.autoresponse.ResponseHandler;
import org.magmafoundation.bot.command.base.CommandManager;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.log.LogUploader;
import org.magmafoundation.bot.reactions.ReactionHandler;
import org.magmafoundation.bot.utils.RoleCache;
import org.slf4j.Logger;

public class BaseEventListener extends ListenerAdapter {

    private final Logger LOGGER = SBLogger.getLogger(BaseEventListener.class);

    public static boolean ready = false;

    public void onReady(@NotNull ReadyEvent event) {
        LOGGER.info("Bot is ready! Running on version: " + AppMain.getVersion().gitHash());
        TextChannel logChannel = event.getJDA().getTextChannelById(Magma.log_channel_id);
        if (logChannel != null)
            logChannel.sendMessage("Bot is ready! Running on version: " + AppMain.getVersion().gitHash()).queue();
        else
            LOGGER.error("Log channel does not exist, not sending message");
        RoleCache.build(event.getJDA());
        LogUploader.checkEnabled();
        ready = true;
    }

    public void onGuildJoin(@NotNull GuildJoinEvent event) {
        LOGGER.info("Bot joined a new guild: " + event.getGuild().getName());
        CommandManager.onGuildJoin(event.getGuild());
    }

    public void onGuildLeave(@NotNull GuildLeaveEvent event) {
        LOGGER.info("Bot left a guild: " + event.getGuild().getName());
        CommandManager.onGuildLeave(event.getGuild());
    }

    public void onMessageReceived(@NotNull MessageReceivedEvent event) {
        if (event.getAuthor().isBot() || event.isWebhookMessage() || !event.isFromGuild())
            return;

        if (!event.getMessage().getAttachments().isEmpty() && LogUploader.enabled)
            event.getMessage().getAttachments().forEach(attachment -> LogUploader.uploadLog(attachment, event));

        ResponseHandler.handle(event);
    }

    public void onMessageDelete(@NotNull MessageDeleteEvent event) {
        if (!event.isFromGuild())
            return;

        CommandManager.getButtonHandler(event.getGuild()).handleRemove(event.getMessageIdLong());
    }

    public void onSlashCommandInteraction(@NotNull SlashCommandInteractionEvent event) {
        CommandManager.handle(event.getChannel().asGuildMessageChannel(), event);
    }

    public void onButtonInteraction(@NotNull ButtonInteractionEvent event) {
        CommandManager.getButtonHandler(event.getGuild()).handle(event);
    }

    //TODO: handle verification
    public void onMessageReactionAdd(@NotNull MessageReactionAddEvent event) {
        if (event.getUser() == null || event.getUser().isBot() || !event.isFromGuild())
            return;

        ReactionHandler handler = CommandManager.getReactionHandler(event.getGuild());

        if (handler.canHandle(event.getGuild().getIdLong(), event.getMessageIdLong()))
            handler.handle(event.getChannel().asTextChannel(), event.getMessageIdLong(), event.getUserIdLong(), event.getReaction());
    }

    public void onRoleDelete(@NotNull RoleDeleteEvent event) {
        RoleCache.build(event.getJDA());
    }

    public void onGuildMemberRoleAdd(@NotNull GuildMemberRoleAddEvent event) {
        event.getRoles().forEach(role -> RoleCache.handle(role, event.getGuild()));
    }

    public void onGuildMemberRoleRemove(@NotNull GuildMemberRoleRemoveEvent event) {
        event.getRoles().forEach(role -> RoleCache.handle(role, event.getGuild()));
    }
}
