package org.magmafoundation.bot.services;

import dev.JustRed23.stonebrick.service.Service;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.gitlab4j.api.GitLabApi;
import org.gitlab4j.api.models.Release;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.magmafoundation.bot.AppMain;
import org.magmafoundation.bot.core.Magma;
import org.magmafoundation.bot.event.BaseEventListener;
import org.magmafoundation.bot.update.AutoUpdate;
import org.magmafoundation.bot.utils.JarUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Consumer;

public class UpdateChecker extends Service {
    private static final Logger LOGGER = LoggerFactory.getLogger(UpdateChecker.class);

    private static boolean updateFound = false;
    private static final AtomicBoolean checking = new AtomicBoolean(false);

    public boolean shouldRun() {
        return BaseEventListener.ready && (AppMain.getGitLabApi() != null) && !updateFound && JarUtils.getJarFile() != null;
    }

    public long delayBetweenRuns() {
        return TimeUnit.MINUTES.toMillis(5);
    }

    public void run() throws Exception {
        checkUpdate();
    }

    public static void checkUpdate() throws Exception{
       checkUpdate(null);
    }
    public static void checkUpdate(@Nullable Consumer<@NotNull String> updateMessage) throws Exception {
        if (checking.getAndSet(true)) {
            if (updateMessage!=null) updateMessage.accept("Already checking for updates.");
            return;
        }
        try {
            GitLabApi api = AppMain.getGitLabApi();
            String repo = Magma.repo_url.replace(Magma.git_url, "");

            if (api == null) {
                if (updateMessage != null) updateMessage.accept("Could not get GitLab API");
                return;
            }

            Release latest = api.getReleasesApi().getReleases(repo).get(0);

            if (latest == null) {
                if (updateMessage != null) updateMessage.accept("Could not get latest release.");
                return;
            }

            if (!AppMain.getVersion().gitHash().equals(latest.getCommit().getShortId().substring(0, 7))) {
                updateFound = true;
                TextChannel logChannel = AppMain.getInstance().getTextChannelById(Magma.log_channel_id);
                if (logChannel != null)
                    logChannel.sendMessage("Update found, bot will restart in a few seconds").queue();
                else
                    LOGGER.error("Log channel does not exist, not sending message");
                if (updateMessage != null) updateMessage.accept("Update found.");
                AutoUpdate.updateFound(latest);
                checking.set(false);
            }
            if (updateMessage != null) updateMessage.accept("No Update found.");
        }finally{
            checking.set(false);
        }
    }
}
