package org.magmafoundation.bot.utils;

import dev.JustRed23.stonebrick.log.SBLogger;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import net.dv8tion.jda.api.requests.restaction.MessageCreateAction;
import net.dv8tion.jda.api.requests.restaction.interactions.ReplyCallbackAction;
import org.magmafoundation.bot.utils.btn.BetterButton;

import java.util.Arrays;
import java.util.List;

public class MessageUtils {

    public static long sendMessageGetID(MessageCreateAction action) {
        try {
            return action.complete().getIdLong();
        } catch (Exception e) {
            SBLogger.getLogger(MessageUtils.class).error("Could not get message ID", e);
            return -1;
        }
    }

    public static long replyGetID(ReplyCallbackAction action) {
        try {
            return action.complete().getInteraction().getIdLong();
        } catch (Exception e) {
            SBLogger.getLogger(MessageUtils.class).error("Could not get reply ID", e);
            return -1;
        }
    }

    public static void messageWithButtons(MessageCreateAction action, Guild guild, BetterButton... buttons) {
        try {
            List<Button> actionRow = Arrays.stream(buttons).map(BetterButton::build).toList();
            long msgID = action.addActionRow(actionRow).complete().getIdLong();
            for (BetterButton button : buttons) //add buttons to the button handler
                button.getButtonHandler(guild).addButton(msgID, button);
        } catch (Exception e) {
            SBLogger.getLogger(MessageUtils.class).error("Could not send message with buttons", e);
        }
    }

    public static void replyWithButtons(ReplyCallbackAction action, Guild guild, BetterButton... buttons) {
        try {
            List<Button> actionRow = Arrays.stream(buttons).map(BetterButton::build).toList();
            long msgID = action.addActionRow(actionRow).complete().retrieveOriginal().complete().getIdLong();
            for (BetterButton button : buttons) //add buttons to the button handler
                button.getButtonHandler(guild).addButton(msgID, button);
        } catch (Exception e) {
            SBLogger.getLogger(MessageUtils.class).error("Could not reply with buttons", e);
        }
    }
}
