package org.magmafoundation.bot.utils;

import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.User;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class RoleCache {

    private static final Map<Guild, Map<Role, List<Member>>> cachedRoles = new ConcurrentHashMap<>();

    public static void build(JDA jda) {
        cachedRoles.clear();

        for (Guild guild : jda.getGuilds()) {
            Map<Role, List<Member>> roles = new ConcurrentHashMap<>();
            for (Role role : guild.getRoles()) {
                roles.put(role, guild.getMembersWithRoles(role));
            }
            cachedRoles.put(guild, roles);
        }
    }

    public static void handle(Role role, Guild guild) {
        if (cachedRoles.containsKey(guild)) {
            Map<Role, List<Member>> roles = cachedRoles.get(guild);
            if (roles.containsKey(role)) {
                roles.remove(role);
                roles.put(role, guild.getMembersWithRoles(role));
            } else {
                roles.put(role, guild.getMembersWithRoles(role));
            }
        } else {
            Map<Role, List<Member>> roles = new ConcurrentHashMap<>();
            roles.put(role, guild.getMembersWithRoles(role));
            cachedRoles.put(guild, roles);
        }
    }

    public static Map<Role, List<Member>> getRoles(Guild guild) {
        return cachedRoles.get(guild);
    }

    public static List<Member> getMembers(Role role, Guild guild) {
        return cachedRoles.get(guild).get(role);
    }

    public static List<User> getUsers(Role role, Guild guild) {
        return getMembers(role, guild).stream().map(Member::getUser).toList();
    }
}
